#from Products.ERP5Type.Cache import CachingMethod
import nltk
import numpy as np
from sklearn.feature_extraction.text import TfidfVectorizer

from Products.ERP5Type.Log import log

stemmer = nltk.stem.porter.PorterStemmer()
def stemTokens(tokens):
  return [stemmer.stem(item) for item in tokens]


def Answer_getCorrectionSuggestion(self, answer_set_list=None, try_cache=True):

  memcached_tool = self.getPortalObject().portal_memcached
  memcached_dict = memcached_tool.getMemcachedDict(key_prefix='correctionSuggestion', plugin_path='portal_memcached/default_memcached_plugin')
  if try_cache:
    try:
      existing_suggestion = memcached_dict[self.getPath()]
      return existing_suggestion
    except KeyError:
      pass
  else:
    log("Answer_getCorrectionSuggestion without caching!")

  def getCorrectionSuggestion(answer_set_list):
    question_id = str(self.getResourceId())

    portal = self.getPortalObject()
    if answer_set_list is None:
      log("Answer_getCorrectionSuggestion without trainingsdata!")
      answer_set_list = portal.portal_catalog(portal_type="Answer Set", validation_state="archived", version="001")

    ## Get ALL the data from ERP and process within this cell so other cells do not depend on the ERP interface
    #  this is due to ERP being slightly unstable and causing trouble

    st = []
    frm = 0
    to = len(answer_set_list)

    example_answer = self

    relevant_answer_list= []
    for i in range(frm, to):
      answer_set = answer_set_list[i].getObject()
      answer_list = answer_set.objectValues(portal_type="Answer")

      for answer in answer_list:
        answer = answer.getObject()

        current_question_id = answer.getResourceId()

        if current_question_id is None:
          continue

        st.append(current_question_id)

        # Compare the id of the current question with the wanted question id and has corrections
        if str(current_question_id) == question_id and len(answer.objectValues(portal_type="Correction Line")) > 0:
          # Get the correction IDs for this answer
          c_id_list = []
          correction_line_list = answer.objectValues(portal_type="Correction Line")
          if correction_line_list != None:
            for c in correction_line_list:
              c_id_list.append(c.getSourceId())

          # Add to the final list
          relevant_answer_list.append((answer_set.getId(), answer, c_id_list, answer.getTextValue()))


    all_correction_list = []
    #texblob_by_correction_list = []
    for answer in relevant_answer_list:
      all_correction_list += answer[2]

    #return "Hello " + str(all_correction_list)

    used_correction_list = sorted([[i, 0, ""] for i in list(set(all_correction_list))])
    for correction in used_correction_list:
      for answer in relevant_answer_list:
        for ac in answer[2]:
          #print str(correction[0]) + " == " + ac
          if str(correction[0]) == ac:
            correction[1] += 1
            correction[2] += answer[3]

    total_corrections = 0
    for c in used_correction_list:
      total_corrections += c[1]

    rating_list = [c[1]/float(total_corrections) for c in used_correction_list]
    final_list = list(zip([c[0] for c in used_correction_list], rating_list))

    return list(reversed(sorted(final_list, key=lambda x: x[1])))

  # Caching. Does not permanetly cache so not used anymore
  #getCorrectionSuggestion = CachingMethod(getCorrectionSuggestion, "getCorrectionSuggestion@" + self.getPath())
  #return getCorrectionSuggestion()
  correctionSuggestion = getCorrectionSuggestion(answer_set_list)
  memcached_dict[self.getPath()] = correctionSuggestion
  return correctionSuggestion


# returns a dict with similar, suggested configurations to the Answer Set this method is called on
# the format of the returned dict is given by the Javascript processing it
def AnswerSet_getSuggestionExampleData(self, answer_set_list=None, try_cache=True):

  memcached_tool = self.getPortalObject().portal_memcached
  memcached_dict = memcached_tool.getMemcachedDict(key_prefix='correctionSuggestion', plugin_path='portal_memcached/default_memcached_plugin')

  if try_cache:
    try:
      existing_suggestion = memcached_dict[self.getPath()]
      return existing_suggestion
    except KeyError:
      pass
  else:
    log("Answer_getSuggestionExampleData without caching!")


  def getSuggestionExampleData(answer_set_list):
    portal = self.getPortalObject()
    answer_set_module = portal.answer_set_module

    new_answer_set = (self.getId(), getTextBlob(self))

    if answer_set_list is None:
      answer_set_list = self.portal_catalog(portal_type="Answer Set", validation_state="released")

    answer_set_list_tb = [(a.getId(), getTextBlob(a)) for a in answer_set_list]
    scoring_list = rateSimilarity(new_answer_set, answer_set_list_tb)[1:]
    ## for testing:
    #scoring_list = [("20131206-26195B8", 0.7), ("20120123-1AD1FB", 0.5)]

    result_dict = {"function": [],
                   "product_line": [],
                   #"group": [],
                   "grade": [],
                   "region": [],
                   #"site": [],
                   "publication_section": [],
                   "role": [],
                   "activity": [],
                   #"nationality": [],
                   "skill": []
                  }
    example_id = 1
    for example in scoring_list:

      answer_set = answer_set_module[example[0]]
      answer_list = answer_set.searchFolder(resource_id=43)
      if(len(answer_list) != 1):
        raise Exception("Answer has no category table though it should!")

      # Sometimes the category_dict is not available due to issues in ERP5 conversion or similar
      # For reference: One example is AnswerSet 20141022-B36EA0
      if not hasattr(answer_list[0], "category_dict"):
        continue

      # should use getter instead
      category_dict = answer_list[0].category_dict

      for base_category in result_dict.keys():
        category_list = []
        result_dict[base_category].append({"example_id": example_id,
                                           "example_score": example[1],
                                           "example_categories": category_list
                                          })

        for entry in category_dict[base_category].values():
          category_list.append({"path": entry["path"],
                                "title": entry["title"],
                                "description": entry.get("description") if entry.get("description") != None else "",
                                "short_title": "",
                                "id": entry["id"]
                               })
        category_list = sortCategoryList(category_list)
        #return [c["path"] for c in category_list]
      example_id += 1

      # We need 5 examples
      if example_id > 5:
        break

    return result_dict

  # For caching -- Update: Using Memcache now
  #getSuggestionExampleData = CachingMethod(getSuggestionExampleData, "getSuggestionExampleData@" + self.getPath())
  #return getSuggestionExampleData()

  exampleSuggestion = getSuggestionExampleData(answer_set_list)
  memcached_dict[self.getPath()] = exampleSuggestion
  return exampleSuggestion


def sortCategoryList(category_list):

  # groups the sub-category-paths beneath the top-level paths
  def groupCategories(level):
    this_level = [c for c in category_list if c["path"].count('/') == level]
    for t in this_level:
      # Moves elements while iterating. Should be changed for clean code
      for c in category_list:
        if cutAfter(level, c["path"]) == t["path"]:
          moveEleBelow(t, c)

  # checks if the level matches as this_level
  def cutAfter(l, s):
    curr = 0
    r = ""
    for c in s:
      if c == '/':
        curr += 1
      if curr > l:
        return r
      r += c
    return ""

  # moves a child-path beneath it's parent
  def moveEleBelow(top, to_move):
    top_index = category_list.index(top)
    old_index = category_list.index(to_move)
    category_list.insert(top_index + 1, category_list.pop(old_index))

  max_level = max([1] + [c["path"].count('/') for c in category_list])
  for i in range(1, max_level):
    groupCategories(i)

  return category_list


# Rate similarity based on
# http://scikit-learn.org/stable/modules/generated/sklearn.feature_extraction.text.TfidfTransformer.html
def rateSimilarity(answer_set_tb, answer_set_list_tb):

  def normalizeText(text):
    interm = "".join(ch for ch in text if ch.isalnum() or ch == ' ')
    return stemTokens(interm.lower().split())

  def cosineSim(text1, text2):
    tfidf = vectorizer.fit_transform([text1, text2])
    return ((tfidf * tfidf.T).A)[0,1]

  vectorizer = TfidfVectorizer(tokenizer=normalizeText, stop_words="english")
  similarity_list = []

  for (_, tb) in answer_set_list_tb:
    similarity_list.append(cosineSim(answer_set_tb[1], tb))

  indexmax_list = np.argsort(similarity_list)
  return list(reversed(zip([answer_set_list_tb[i][0] for i in indexmax_list], [similarity_list[i] for i in indexmax_list])))


# Pack all answers of text-questions in one string
def getTextBlob(answer_set):
  answer_list = answer_set.objectValues(portal_type="Answer")
  return "".join([a.getTextValue() for a in answer_list])