##############################################################################
#
# Copyright (c) 2002-2016 Nexedi SA and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
##############################################################################

from Products.ERP5Type.tests.ERP5TypeTestCase import ERP5TypeTestCase

class Test(ERP5TypeTestCase):
  """
  Test for SurveyMachineLearning ie. exampleSuggestion and correctionSuggestion
  """

  def getTitle(self):
    return "SurveyExampleMachineLearningTest"

  def getBusinessTemplateList(self):
    return ('erp5_survey_ml',)


  def afterSetUp(self):
    self.portal = self.getPortalObject()
    self.test_id_list = ["20141016-14ABCCD", "20130310-A7476B", "20130228-55CF76", "20130301-254E01", "20120817-6464C7", "20141104-29F35D6"]
    self.test_set_list = [self.portal.answer_set_module[idd].Base_createCloneDocument(batch_mode=1) for idd in self.test_id_list]
    #self.test_set_list = [self.portal.answer_set_module[idd] for idd in self.test_id_list]
    self.tic()

  def beforeTearDown(self):
    self.portal.answer_set_module.manage_delObjects([ans.getId() for ans in self.test_set_list])
    self.tic()
    pass


  #
  #
  #
  # Tests
  #
  #
  #


  def test_01_checkOutput(self):
    answer_set = self.test_set_list[0]
    result = answer_set.AnswerSet_getSuggestionExampleData(answer_set_list=self.test_set_list, try_cache=False)
    self.assertEqual([r["example_score"] for r in result["function"]], self.getRef(self.test_id_list[0]))

  def test_02_checkOutput(self):
    answer_set = self.test_set_list[1]
    result = answer_set.AnswerSet_getSuggestionExampleData(answer_set_list=self.test_set_list, try_cache=False)
    self.assertEqual([r["example_score"] for r in result["function"]], self.getRef(self.test_id_list[1]))

  def test_03_checkOutput(self):
    answer_set = self.test_set_list[2]
    result = answer_set.AnswerSet_getSuggestionExampleData(answer_set_list=self.test_set_list, try_cache=False)
    self.assertEqual([r["example_score"] for r in result["function"]], self.getRef(self.test_id_list[2]))

  def test_04_checkOutput(self):
    answer_set = self.test_set_list[3]
    result = answer_set.AnswerSet_getSuggestionExampleData(answer_set_list=self.test_set_list, try_cache=False)
    self.assertEqual([r["example_score"] for r in result["function"]], self.getRef(self.test_id_list[3]))

  def test_05_testCaching(self):
    memcached_tool = self.getPortalObject().portal_memcached
    memcached_dict = memcached_tool.getMemcachedDict(key_prefix='correctionSuggestion', plugin_path='portal_memcached/default_memcached_plugin')

    answer_set = self.test_set_list[1]
    try:
      existing_suggestion = memcached_dict[answer_set.getPath()]
      memcached_dict[answer_set.getPath()] = None
    except KeyError:
      pass

    uncachedResults = answer_set.AnswerSet_getSuggestionExampleData(answer_set_list=self.test_set_list, try_cache=True)

    try:
      existing_suggestion = memcached_dict[answer_set.getPath()]
    except KeyError:
      self.fail("Could not find memcached entry")

  #
  #
  #
  #
  #
  #
  # Reference Solutions
  #
  #
  #
  #
  #


  def getRef(self, answer_set_id):
    ref = {
      "20141016-14ABCCD": [0.27691720337528553, 0.22155241429893047, 0.21376222493678282, 0.18730902818146625],
      "20130310-A7476B": [0.4487432151107279, 0.42363015409455379, 0.41364903296494099],
      "20130228-55CF76": [0.44874321511072796, 0.44140235717688475, 0.26087844801894289],
      "20130301-254E01": [0.44140235717688497, 0.42363015409455379, 0.32496375159721441]
    }
    return ref[answer_set_id]
